#include <genieArduino.h>
#include <SoftwareSerial.h>

#define RESETLINE 4   // Change this if you are not using an Arduino Adaptor Shield Version 2 (see code below)
#define READ_OBJ 0    
#define WRITE_OBJ 1

// Application Notes on the 4D Systems Website that are useful to understand this library are found: http://www.4dsystems.com.au/appnotes
// Good App Notes to read are:
// ViSi-Genie Connecting a 4D Display to an Arduino Host - http://www.4dsystems.com.au/appnote/4D-AN-00017/
// ViSi-Genie Writing to Genie Objects Using an Arduino Host - http://www.4dsystems.com.au/appnote/4D-AN-00018/
// ViSi-Genie A Simple Digital Voltmeter Application using an Arduino Host - http://www.4dsystems.com.au/appnote/4D-AN-00019/
// ViSi-Genie Connection to an Arduino Host with RGB LED Control - http://www.4dsystems.com.au/appnote/4D-AN-00010/
// ViSi-Genie Displaying Temperature values from an Arduino Host - http://www.4dsystems.com.au/appnote/4D-AN-00015/
// ViSi-Genie Arduino Danger Shield - http://www.4dsystems.com.au/appnote/4D-AN-00025/

Genie genie;

// Use software serial on pins 10 and 11, which means set the RX and TX jumpers on
// the Arduino breakout board to use D10 and D11.
SoftwareSerial serial(10, 11);

String inputString = "";         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete

byte messageType;
int messageIndex;
int messageValue;

void setup()
{
  // NOTE, the genieBegin function (e.g. genieBegin(GENIE_SERIAL_0, 115200)) no longer exists.
  Serial.begin(9600);  // Use Serial0 to communicate with the Serial Monitor of the Arduino IDE

  serial.begin(9600);  //
  genie.Begin(serial); // Use the software serial port to communicate with the display module

  genie.AttachEventHandler(myGenieEventHandler); // Attach the user function Event Handler for processing events

  // Reset the Display (change D4 to D2 if you have original 4D Arduino Adaptor)
  // THIS IS IMPORTANT AND CAN PREVENT OUT OF SYNC ISSUES, SLOW SPEED RESPONSE ETC
  // If NOT using a 4D Arduino Adaptor, digitalWrites must be reversed as Display Reset is Active Low, and
  // the 4D Arduino Adaptors invert this signal so must be Active High.
  pinMode(RESETLINE, OUTPUT);  // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(RESETLINE, 1);  // Reset the Display via D4
  delay(100);
  digitalWrite(RESETLINE, 0);  // unReset the Display via D4

  delay (3500); //let the display start up after the reset (This is important)

  //Turn the Display on (Contrast) - (Not needed but illustrates how)
  genie.WriteContrast(1); // 1 = Display ON, 0 = Display OFF.
  //For uLCD43, uLCD-70DT, and uLCD-35DT, use 0-15 for Brightness Control, where 0 = Display OFF, though to 15 = Max Brightness ON.

  inputString.reserve(200); // reserve 200 bytes for the inputString
  Serial.println("Warning: Do not write to or read from objects \nthat do not exist on the display program.");
  Serial.println("Waiting for a command...");  
}

void loop()
{
  static long waitPeriod = millis();
  
  genie.DoEvents(); // This calls the library each loop to process the queued responses from the display

  if (millis() >= waitPeriod)
  {
    serialEvent(); // get an input message or command from the user
    if (stringComplete) {     
      if(evaluateMessage(inputString)){ // if the input string is a valid command or message
        if(messageType == READ_OBJ){  
          Serial.print("\n");               
          Serial.print("Reading Dipswitch");
          Serial.print(messageIndex);
          Serial.println(" (READ_OBJ)...");
          genie.ReadObject(GENIE_OBJ_DIPSW, messageIndex);
        }
        else if(messageType == WRITE_OBJ){
          Serial.print("Dipswitch");
          Serial.print(messageIndex);
          Serial.print(" set to ");
          Serial.print(messageValue);
          Serial.println(" (WRITE_OBJ).");
          genie.WriteObject(GENIE_OBJ_DIPSW, messageIndex, messageValue);
        }
      }

      else{
        Serial.print(inputString);
        Serial.println(" is not a valid command. Please try again.");      
      }    
      
      inputString = ""; // clear the string:
      stringComplete = false;
    }

    waitPeriod = millis() + 50; // rerun this block in another 50ms time.
  }
}

/////////////////////////////////////////////////////////////////////
//
// This is the user's event handler. It is called by genieDoEvents()
// when the following conditions are true
//
//		The link is in an IDLE state, and
//		There is an event to handle
//
// The event can be either a REPORT_EVENT frame sent asynchronously
// from the display or a REPORT_OBJ frame sent by the display in
// response to a READ_OBJ request.
//

// LONG HAND VERSION
void myGenieEventHandler(void)
{
  genieFrame Event;
  genie.DequeueEvent(&Event);

  int dipSwitch_val = 0;

  //If the cmd received is from a Reported Event or REPORT_EVENT (Events triggered from the Events tab of Workshop4 objects)
  if (Event.reportObject.cmd == GENIE_REPORT_EVENT)
  {
    if (Event.reportObject.object == GENIE_OBJ_DIPSW)                // If the REPORT_EVENT Message was from a DIP switch
    {
      dipSwitch_val = genie.GetEventData(&Event);                     // Receive the event data from the DIP switch object

      if (Event.reportObject.index == 0) {                             // If from Dipswitch0
        Serial.print("Dipswitch0: ");
        Serial.print(dipSwitch_val);
        Serial.println(" (REPORT_EVENT)");
      }

      else if (Event.reportObject.index == 1) {                        // If from Dipswitch1
        Serial.print("Dipswitch1: ");
        Serial.print(dipSwitch_val);
        Serial.println(" (REPORT_EVENT)");
      }

      //else
        //unhandled event
      }
  }

  //If the cmd received is from a Reported Object (or REPORT_OBJ), which occurs if a Read Object (genie.ReadOject or READ_OBJ) is requested in the main code, reply processed here.
  if (Event.reportObject.cmd == GENIE_REPORT_OBJ)
  {
    if (Event.reportObject.object == GENIE_OBJ_DIPSW)                // If the REPORT_EVENT Message was from a DIP switch
    {
      dipSwitch_val = genie.GetEventData(&Event);                     // Receive the event data from the DIP switch object
      //Serial.print("\n");
      if (Event.reportObject.index == 0) {                             // If from Dipswitch0
        Serial.print("Queried value of Dipswitch0: ");
        Serial.print(dipSwitch_val);
        Serial.println(" (REPORT_OBJ)");
      }

      else if (Event.reportObject.index == 1) {                        // If from Dipswitch1
        Serial.print("Queried value of Dipswitch1: ");
        Serial.print(dipSwitch_val);
        Serial.println(" (REPORT_OBJ)");
      }

      else if (Event.reportObject.index == 2) {                        // If from Dipswitch2
        Serial.print("Queried value of Dipswitch2: ");
        Serial.print(dipSwitch_val);
        Serial.println(" (REPORT_OBJ)");
      }

      //else
        //unhandled event
      }
  }

  //This can be expanded as more objects are added that need to be captured

  //Event.reportObject.cmd is used to determine the command of that event, such as an reported event
  //Event.reportObject.object is used to determine the object type, such as a Slider
  //Event.reportObject.index is used to determine the index of the object, such as Slider0
  //genie.GetEventData(&Event) us used to save the data from the Event, into a variable.
}

/*
  SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 */
void serialEvent() {
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n') {
      inputString.trim();
      stringComplete = true;
    }
  }
}

// this routine was written for demonstration purposes only
boolean evaluateMessage(String inputMessage){
  if (inputMessage.length() < 6 || inputMessage.length() > 18 ) return false;
  boolean returnFlag = true;
  
  //if inputMessage has the substring "read"
  if(inputMessage.substring(0,4) == "read"){
    messageType = READ_OBJ;
    messageIndex = getIndex(inputMessage);
  }

  //else if inputMessage has the substring "write"
  else if(inputMessage.substring(0,5) == "write"){  
    messageType = WRITE_OBJ;
    messageIndex = getIndex(inputMessage);
    messageValue = getValue(inputMessage);
  }

  else
    returnFlag = false;

  return returnFlag;
}

int getIndex(String message){
  static char indexChars[6];
  static char messageBuffer[19];
  int firstSpace, secondSpace;
  
  memset(indexChars,0,sizeof(indexChars));  // clear array
  memset(messageBuffer,0,sizeof(messageBuffer));  // clear array  
  
  firstSpace = message.indexOf(' ');
  secondSpace = message.indexOf(' ', firstSpace + 1);
  if(secondSpace < 0 || secondSpace > message.length() - 1)
    secondSpace = message.length();
     
  message.toCharArray(messageBuffer, sizeof(messageBuffer));
  
  strncpy(indexChars, messageBuffer + firstSpace + 1 , secondSpace - (firstSpace + 1)); 
  return atoi(indexChars);   
}

int getValue(String message){
  static char valueChars[6];
  static char messageBuffer[19];
  int firstSpace, secondSpace;
  
  memset(valueChars,0,sizeof(valueChars));  // clear array
  memset(messageBuffer,0,sizeof(messageBuffer));  // clear array  
  
  firstSpace = message.indexOf(' ');
  secondSpace = message.indexOf(' ', firstSpace + 1);
  message.toCharArray(messageBuffer, sizeof(messageBuffer));
  
  strncpy(valueChars, messageBuffer + secondSpace + 1 , message.length() - (secondSpace+1)); 
  return atoi(valueChars); 
}


